#######################################################################
# cell.py
#######################################################################
"""
   generate basic cell block for memory array
"""

# PyCell code
from cni.dlo import (Box, DloGen, Instance, ParamArray, Point, Rect, Term, Tech, Layer, TermType, Pin, PropSet, InstanceArray, PhysicalComponent, Text, Orientation)
import math

##### Technology definition #####
tech = Tech.get('NCSU_TechLib_FreePDK3D45')
grid = 0.0025
nfet_minlength=tech.getMosfetParams('nmos_vtl', 'thin', 'minLength')
pfet_minlength=tech.getMosfetParams('pmos_vtl', 'thin', 'minLength')
nfet_minwidth=tech.getMosfetParams('nmos_vtl', 'thin', 'minWidth')
pfet_minwidth=tech.getMosfetParams('pmos_vtl', 'thin', 'minWidth')
poly_active_enc=tech.getPhysicalRule('minEnclosure', tech.getLayer('poly'), tech.getLayer('active'))
poly_contact_enc=tech.getPhysicalRule('minEnclosure', tech.getLayer('poly'), tech.getLayer('contact'))
poly_contact_spa=tech.getPhysicalRule('minSpacing', tech.getLayer('poly'), tech.getLayer('contact'))
active_contact_enc=tech.getPhysicalRule('minEnclosure', tech.getLayer('active'), tech.getLayer('contact'))
nwell_active_enc=tech.getPhysicalRule('minEnclosure', tech.getLayer('nwell'), tech.getLayer('active'))
contact_minwidth=tech.getPhysicalRule('minWidth', tech.getLayer('contact'))
#print contact_minwidth
metal1_minwidth=tech.getPhysicalRule('minWidth', tech.getLayer('metal1'))
metal1_minspacing=tech.getPhysicalRule('minSpacing', tech.getLayer('metal1'))
metal2_minwidth=tech.getPhysicalRule('minWidth', tech.getLayer('metal2'))
metal2_minspacing=tech.getPhysicalRule('minSpacing', tech.getLayer('metal2'))
metal3_minwidth=tech.getPhysicalRule('minWidth', tech.getLayer('metal3'))
metal3_minspacing=tech.getPhysicalRule('minSpacing', tech.getLayer('metal3'))


# Gate height & width calculation
# gate_height=contact_minwidth + poly_contact_enc*2 +poly_active_enc*2 + nfet_minwidth*NW_nfet + pfet_minwidth*NW_pfet + metal1_minspacing*2 + metal1_minwidth*2
# gate_width=nwell_active_enc*2 + active_contact_enc*2 + (max(nfet_minlength*NL_nfet, pfet_minlength*NL_pfet) + poly_contact_spa*2 + contact_minwidth)*N_finger + contact_minwidth

##### Global parameters definition #######
## SRAM cell area to gate length coefficient, based on ITRS statistic
cell_coe = 100
## cell_area = 0.49
cell_area = nfet_minlength*pfet_minlength * cell_coe
cell_ratio = 0.25
cell_width = round( math.sqrt( cell_area / cell_ratio ) / grid) * grid
cell_height = round((cell_width * cell_ratio) / grid) * grid

#### Tier choice: 'A', 'B', 'C', 'D', 'E' or 'All'
#tier_name = 'All'
tier_name = 'A', 'B', 'C', 'D'
#Ntier = 5
Ntier = 4
# tiers in one sub-array (1 means sub-arrays vertically stacking are treated as individual sub-arrays in a bank and selected by control signal)
tier_sub = 1

if tier_name == 'All':
    Ntier = 5
if Ntier == 3 or Ntier == 5:
    tier_sub_NOB = int(round(math.log(Ntier/tier_sub, 2)))+1
else:
    tier_sub_NOB = int(round(math.log(Ntier/tier_sub, 2)))
#print 'tier_sub_NOB ='+str(tier_sub_NOB)
# output bits in each sub-array
output_bit = 8
# row number of bit in sub-array
rows_cell_NOB = 9
# col number of bit in sub-array
cols_cell_NOB = 5
# number of rows in sub-array
rows_cell=2**rows_cell_NOB
# number of cols in sub-array
cols_cell=2**cols_cell_NOB*output_bit/tier_sub
# bit-line number per sense-amp
bl_per_sa = 2**cols_cell_NOB
# sub-array rows in bank
rows_sub_NOB = 5
rows_sub=2**rows_sub_NOB
# sub-array cols in bank
cols_sub_NOB = 2
cols_sub=2**cols_sub_NOB
# channel width in bank level for wires (In metal2)
wire_bank = (output_bit/tier_sub)*cols_sub*(metal2_minwidth+metal2_minspacing)+(rows_cell_NOB+cols_cell_NOB)*(metal2_minwidth+metal2_minspacing)

# Number of pins in sub-array, bank
pins_clk = 1
pins_wren = 2
pins_sub = rows_cell_NOB + cols_cell_NOB + (output_bit/tier_sub)*2 + pins_clk + pins_wren
pins_bank = rows_sub_NOB + cols_sub_NOB + tier_sub_NOB + rows_cell_NOB + cols_cell_NOB + output_bit*2 + pins_clk + pins_wren

# bank rows in top
rows_top_NOB = 1
rows_top=2**rows_top_NOB
# bank cols in top
cols_top_NOB = rows_top_NOB
cols_top=2**cols_top_NOB
# Number of pins in top
pins_top = rows_sub_NOB + cols_sub_NOB + tier_sub_NOB + rows_cell_NOB + cols_cell_NOB + output_bit*2*rows_top*cols_top + pins_clk + pins_wren
# channel width in top level for wires next to one bank (In metal3)
wire_top = pins_bank*(metal3_minwidth+metal3_minspacing)

#### I/O pin type: A(VTT), B(VIA_Stack) or C(VIA23)
IO_type_bank = 'B'
IO_type_top = 'A'
#### I/O pin tier: 'B' to 'E' (VTT); 'A' to 'E' (VIA_Stack & VIA23)
IO_tier_bank = 'A'
IO_tier_top = 'B'
#### position of top level IO: E(dge) or C(enter) (default)
IO_position_top = 'C'

# number of IO rows in bank level
IO_bank_rows=4
# number of IO rows in top level
#IO_top_rows = int(round(math.sqrt(pins_top)))
IO_top_rows = 8

## local word-line driver
#width_ld = 1.0
# may need to extend with col number in cell array
width_ld = nwell_active_enc*2 + active_contact_enc*2 + (max(nfet_minlength*1, pfet_minlength*1) + poly_contact_spa*2 + contact_minwidth)*10 + contact_minwidth
height_ld = cell_height

## row decoder
#width_rdec = 1.4
# may need to extend with col number & tiers in cell array
width_rdec = nwell_active_enc*2 + active_contact_enc*2 + (max(nfet_minlength*1, pfet_minlength*1) + poly_contact_spa*2 + contact_minwidth)*15 + contact_minwidth
height_rdec = cell_height

## bit-lne precharge and equalization
width_prec = cell_width
#height_prec = 1.0
# may need to extend with row number in cell array
height_prec = contact_minwidth + poly_contact_enc*2 +poly_active_enc*2 + nfet_minwidth*0 + pfet_minwidth*4 + metal1_minspacing*2 + metal1_minwidth*0

## bit-line mutiplexer
width_mux = cell_width * bl_per_sa
#height_mux = 1.4
# may need to extend with row number in cell array
height_mux = contact_minwidth + poly_contact_enc*2 +poly_active_enc*2 + nfet_minwidth*6 + pfet_minwidth*0 + metal1_minspacing*(bl_per_sa+1) + metal1_minwidth*bl_per_sa

## sense-amplifier
width_sa = width_mux
#height_sa = 1.0
height_sa = contact_minwidth + poly_contact_enc*2 +poly_active_enc*2 + nfet_minwidth*2 + pfet_minwidth*4 + metal1_minspacing*2 + metal1_minwidth*2

## write driver
width_wd = width_mux
#height_wd = 0.8
height_wd = contact_minwidth + poly_contact_enc*2 +poly_active_enc*2 + nfet_minwidth*2 + pfet_minwidth*2 + metal1_minspacing*2 + metal1_minwidth*2


#### Via pitch calculation
# VTT pitch calculation
VTT_pitch=tech.getPhysicalRule('minWidth', tech.getLayer('VTT'))+ max(tech.getPhysicalRule('minEnclosure', tech.getLayer('TM'), tech.getLayer('VTT'))*2 + tech.getPhysicalRule('minSpacing', tech.getLayer('TM')), tech.getPhysicalRule('minSpacing', tech.getLayer('VTT')))
VTT_size=tech.getPhysicalRule('minWidth', tech.getLayer('VTT'))+ tech.getPhysicalRule('minEnclosure', tech.getLayer('TM'), tech.getLayer('VTT'))*2
# VIA_Stack (VUP) pitch calculation
VUP_pitch=tech.getPhysicalRule('minWidth', tech.getLayer('VUP'))+ max(tech.getPhysicalRule('minEnclosure', tech.getLayer('TM'), tech.getLayer('VUP'))*2 + tech.getPhysicalRule('minSpacing', tech.getLayer('TM')), tech.getPhysicalRule('minSpacing', tech.getLayer('VUP')))
VUP_size=tech.getPhysicalRule('minWidth', tech.getLayer('VUP'))+ tech.getPhysicalRule('minEnclosure', tech.getLayer('TM'), tech.getLayer('VUP'))*2
# VIA_23 pitch calculation
V23_pitch=tech.getPhysicalRule('minWidth', tech.getLayer('via2'))+ max(tech.getPhysicalRule('minEnclosure', tech.getLayer('metal3'), tech.getLayer('via2'))*2 + tech.getPhysicalRule('minSpacing', tech.getLayer('metal3')), tech.getPhysicalRule('minSpacing', tech.getLayer('via2')))
V23_size=tech.getPhysicalRule('minWidth', tech.getLayer('via2'))+ tech.getPhysicalRule('minEnclosure', tech.getLayer('metal3'), tech.getLayer('via2'))*2

## difine bank IO Via type
if IO_type_bank == 'A':
   Via_pitch_bank = VTT_pitch
   Via_name_bank = 'VTT_'+IO_tier_bank
   Via_size_bank = VTT_size
elif IO_type_bank == 'B':
   Via_pitch_bank = VUP_pitch
   Via_name_bank = 'VIA_Stack_'+IO_tier_bank
   Via_size_bank = VUP_size
else:
   Via_pitch_bank = V23_pitch
   Via_name_bank = 'VIA23_'+IO_tier_bank
   Via_size_bank = V23_size

## difine top IO Via type
if IO_type_top == 'A':
   Via_pitch_top = VTT_pitch
   Via_name_top = 'VTT_'+IO_tier_top
   Via_size_top = VTT_size
elif IO_type_top == 'B':
   Via_pitch_top = VUP_pitch
   Via_name_top = 'VIA_Stack_'+IO_tier_top
   Via_size_top = VUP_size
else:
   Via_pitch_top = V23_pitch
   Via_name_top = 'VIA23_'+IO_tier_top
   Via_size_top = V23_size

# Top I/O pins generations, added at Jan 2010, xchen
data_width=output_bit*rows_top*cols_top
#addr_width=rows_top_NOB + cols_top_NOB + rows_sub_NOB + cols_sub_NOB + rows_cell_NOB + cols_cell_NOB
addr_width=rows_sub_NOB + cols_sub_NOB + rows_cell_NOB + cols_cell_NOB + tier_sub_NOB

def pingen():
    Pin_top = []
    for i in range(addr_width):
        Pin_top.append('rd_addr1['+str(i)+']')
    for i in range(data_width):
        Pin_top.append('rd_data1['+str(i)+']')
    for i in range(data_width):
        Pin_top.append('wr_data['+str(i)+']')
    Pin_top.append('wr_en')
    Pin_top.append('rd_en1')
    Pin_top.append('clock_rw')
    return Pin_top


# Class defines my PyCell
#
class squ( DloGen):

    @classmethod
    def defineParamSpecs( cls, specs):
        #specs("view", "RTL" )
        specs("area", 1.0)
        specs("aspectRatio", 1.0)

    def setupParams( self, params):    	  
    	  # No "view" in this case
    	  
    	  # Set width and height to ensure vertices fall on layout grid
        self.grid = self.tech.getGridResolution()
        self.width  = round( math.sqrt( params["area" ] / params["aspectRatio"] ) / self.grid) * self.grid
        self.height = round( (self.width * params["aspectRatio"]) / self.grid) * self.grid

        self.blockLayers = [
            self.tech.getLayer("prBoundary"),
            ]
        
    def genLayout( self):
    	  for lyr in self.blockLayers:
            Rect( lyr, Box( 0, 0, self.width, self.height))


    @classmethod
    def unitTest( cls, paramsMaker, lib, cell, view, ignoreError=True):
        pass

class squ3d(squ):

    @classmethod
    def defineParamSpecs( cls, specs):
        squ.defineParamSpecs( specs)
        
        # Add additional parameters and override defaults
#        specs("TIER", "none")
#        specs("TIER", tier_name)
#        specs("Rows", 1)
#        specs("Cols", 1)

    def setupParams( self, params):
        squ.setupParams(self,params)

        # Save parameters needed to calculate power
#        if params[ "TIER" ] != 'All':
        if Ntier == 1:
#            self.tier = params[ "TIER" ]
            self.tier = tier_name            
            self.blockLayers = [
                self.tech.getLayer("metal1_"+self.tier[0]),
#                self.tech.getLayer("prBoundary"),
                ]
        elif Ntier == 2:
            self.tier = tier_name            
            self.blockLayers = [
                self.tech.getLayer("metal1_"+self.tier[0]),
                self.tech.getLayer("metal1_"+self.tier[1]),
#                self.tech.getLayer("prBoundary"),
                ]
        elif Ntier == 3:
            self.tier = tier_name            
            self.blockLayers = [
                self.tech.getLayer("metal1_"+self.tier[0]),
                self.tech.getLayer("metal1_"+self.tier[1]),
                self.tech.getLayer("metal1_"+self.tier[2]),
#                self.tech.getLayer("prBoundary"),
                ]
        elif Ntier == 4:
            self.tier = tier_name            
            self.blockLayers = [
                self.tech.getLayer("metal1_"+self.tier[0]),
                self.tech.getLayer("metal1_"+self.tier[1]),
                self.tech.getLayer("metal1_"+self.tier[2]),
                self.tech.getLayer("metal1_"+self.tier[3]),
#                self.tech.getLayer("prBoundary"),
                ]                
        else:
            self.tier = ('A','B','C','D','E')
            self.blockLayers = [
                self.tech.getLayer("metal1_"+self.tier[0]),
                self.tech.getLayer("metal1_"+self.tier[1]),
                self.tech.getLayer("metal1_"+self.tier[2]),
                self.tech.getLayer("metal1_"+self.tier[3]),
                self.tech.getLayer("metal1_"+self.tier[4]),
#                self.tech.getLayer("prBoundary"),
                ]
        #self.setNumRows = params[ "Rows" ]
        #self.setNumCols = params[ "Cols" ]

        #self.blockLayers = [              
        #    self.tech.getLayer("metal1_"+self.tier),  
        #    self.tech.getLayer("prBoundary"),
        #    self.tech.getLayer("metal2_"+self.tier),
        #    self.tech.getLayer("metal3_"+self.tier),
        #    ]            

        #self.nspinlayer=self.tech.getLayer('metal2')
        #self.ewpinlayer=self.tech.getLayer('metal3')


    def genLayout( self):
        squ.genLayout(self)
        
        print 'Executing squ3d.genLayout()'
        #self.props['POWERLAYERS']='1 ACT_'+self.tier+' 1.0'


################# Modules #################
class cell_new(squ3d):

    @classmethod
    def defineParamSpecs( cls, specs):
        squ3d.defineParamSpecs( specs)
        
        # Add additional parameters and override defaults
        specs.remove("area")
        specs("area", cell_area)
        #specs("area", cell_width*cell_height)
        specs.remove("aspectRatio")
        specs("aspectRatio", cell_ratio)
        #specs("aspectRatio", cell_height/cell_width)
        #specs("xactPerCycle", 0.17)

    def setupParams( self, params):
        squ3d.setupParams(self,params)
        
    def genLayout( self):
        squ3d.genLayout(self)
        
        #setDX(self.width)
        #setDY(self.height)
        #setNumRows( params[ "Rows" ] )
        #setNumCols( params[ "Cols" ] )

class ld(squ3d):

    @classmethod
    def defineParamSpecs( cls, specs):
        squ3d.defineParamSpecs( specs)
        
        # Add additional parameters and override defaults
        #specs.remove("area")
        #specs("area",  0.49)
        #width_ld = nwell_active_enc*2 + active_contact_enc*2 + (max(nfet_minlength*1, pfet_minlength*1) + poly_contact_spa*2 + contact_minwidth)*10 + contact_minwidth
        specs("width", width_ld)
        specs("height", height_ld)

    def setupParams( self, params):
        squ3d.setupParams(self,params)
        
        self.width = params[ "width" ]
        self.height = params[ "height" ]
        
    def genLayout( self):
        squ3d.genLayout(self)
        text1 = Text(self.tech.getLayer("text"),'ld',Point(0.0,height_ld/2),height_ld/2)

class rdec(squ3d):

    @classmethod
    def defineParamSpecs( cls, specs):
        squ3d.defineParamSpecs( specs)
        
        # Add additional parameters and override defaults
        #specs.remove("area")
        #specs("area",  0.49)
        specs("width", width_rdec)
        specs("height", height_rdec)

    def setupParams( self, params):
        squ3d.setupParams(self,params)
        
        self.width = params[ "width" ]
        self.height = params[ "height" ]
        
    def genLayout( self):
        squ3d.genLayout(self)
        text1 = Text(self.tech.getLayer("text"),'rdec',Point(0.0,height_rdec/2),height_rdec/2)

class prec(squ3d):

    @classmethod
    def defineParamSpecs( cls, specs):
        squ3d.defineParamSpecs( specs)
        
        # Add additional parameters and override defaults
        #specs.remove("area")
        #specs("area",  0.49)
        specs("width", width_prec)
        specs("height", height_prec)

    def setupParams( self, params):
        squ3d.setupParams(self,params)
        
        self.width = params[ "width" ]
        self.height = params[ "height" ]
        
    def genLayout( self):
        squ3d.genLayout(self)
        text1 = Text(self.tech.getLayer("text"),'prec',Point(0.0,height_prec/2),width_prec/2)

class mux(squ3d):

    @classmethod
    def defineParamSpecs( cls, specs):
        squ3d.defineParamSpecs( specs)
        
        # Add additional parameters and override defaults
        #specs.remove("area")
        #specs("area",  0.49)
        specs("width", width_mux)
        specs("height", height_mux)

    def setupParams( self, params):
        squ3d.setupParams(self,params)
        
        self.width = params[ "width" ]
        self.height = params[ "height" ]
        
    def genLayout( self):
        squ3d.genLayout(self)
        text1 = Text(self.tech.getLayer("text"),'mux',Point(0.0,height_mux/2),height_mux/2) 

class sa(squ3d):

    @classmethod
    def defineParamSpecs( cls, specs):
        squ3d.defineParamSpecs( specs)
        
        # Add additional parameters and override defaults
        #specs.remove("area")
        #specs("area",  0.49)
        specs("width", width_sa)
        specs("height", height_sa)

    def setupParams( self, params):
        squ3d.setupParams(self,params)
        
        self.width = params[ "width" ]
        self.height = params[ "height" ]
        
    def genLayout( self):
        squ3d.genLayout(self)
        text1 = Text(self.tech.getLayer("text"),'sa',Point(0.0,height_sa/2),height_sa/2)

class wd(squ3d):

    @classmethod
    def defineParamSpecs( cls, specs):
        squ3d.defineParamSpecs( specs)
        
        # Add additional parameters and override defaults
        #specs.remove("area")
        #specs("area",  0.49)
        specs("width", width_wd)
        specs("height", height_wd)

    def setupParams( self, params):
        squ3d.setupParams(self,params)
        
        self.width = params[ "width" ]
        self.height = params[ "height" ]
        
    def genLayout( self):
        squ3d.genLayout(self)
        text1 = Text(self.tech.getLayer("text"),'wd',Point(0.0,height_wd/2),height_wd/2)

######### for array generation ##############

class cell_array(cell_new):

    @classmethod
    def defineParamSpecs( cls, specs):
        cell_new.defineParamSpecs( specs)
        
        specs("Rows", rows_cell)
        specs("Cols", cols_cell)

    def setupParams( self, params):
        cell_new.setupParams(self,params)
        
        self.rows = params[ "Rows" ]
        self.cols = params[ "Cols" ]
        
    def genLayout( self):
    	  #cell.genLayout(self)
    	  #makeArray(self.width, self.height, self.rows, self.cols)
    	  p = ParamArray()
    	  
    	  inst = InstanceArray('py_lib/cell_new', self.width, self.height, self.rows, self.cols, p, None, 'cell_s')
    	  inst.setOrigin(Point(0.0, 0.0))
    	  inst2 = InstanceArray('py_lib/ld', width_ld, self.height, self.rows, 1, p, None, 'ldriver')
    	  inst2.setOrigin(Point(-width_ld, 0.0))
    	  inst3 = InstanceArray('py_lib/rdec', width_rdec, self.height, self.rows, 1, p, None, 'row_dec')
    	  inst3.setOrigin(Point(-width_rdec-width_ld, 0.0))
    	  inst4 = InstanceArray('py_lib/prec', self.width, height_prec, 1, self.cols, p, None, 'prec_equ')
    	  inst4.setOrigin(Point(0.0, self.height*self.rows))
    	  inst5 = InstanceArray('py_lib/mux', self.width*bl_per_sa, height_mux, 1, self.cols/bl_per_sa, p, None, 'mux')
    	  inst5.setOrigin(Point(0.0, -height_mux))
    	  inst6 = InstanceArray('py_lib/sa', self.width*bl_per_sa, height_sa, 1, self.cols/bl_per_sa, p, None, 'sa')
    	  inst6.setOrigin(Point(0.0, -height_mux-height_sa))
    	  inst7 = InstanceArray('py_lib/wd', self.width*bl_per_sa, height_wd, 1, self.cols/bl_per_sa, p, None, 'wd')
    	  inst7.setOrigin(Point(0.0, -height_mux-height_sa-height_wd))

height_sub = height_prec + cell_height*rows_cell + height_mux + height_sa + height_wd+wire_bank
width_sub = width_rdec + width_ld + cell_width*cols_cell+wire_bank

## define bank IO dimension
IO_col_f= pins_bank*1.0 / IO_bank_rows
#print 'IO_col_f ='+str(IO_col_f)
IO_col_i=int(round(IO_col_f))
if IO_col_i == IO_col_f:
	IO_bank_cols=IO_col_i
else:
	IO_bank_cols=IO_col_i+1
	#IO_col_r= pins_bank - IO_col_i*IO_bank_rows	

Coord_X_IO = (Via_pitch_bank * (IO_bank_cols-1) + Via_size_bank)/2
Coord_Y_IO = (Via_pitch_bank * (IO_bank_rows-1) + Via_size_bank)/2
        
class bank(squ3d):

    @classmethod
    def defineParamSpecs( cls, specs):
        squ3d.defineParamSpecs( specs)

        specs("IO_type", IO_type_bank)
        #specs("Rows", IO_bank_rows)
        #specs("Cols", IO_bank_cols)

    def setupParams( self, params):
        squ3d.setupParams(self,params)
#        if params[ "TIER" ] != 'All':
        if Ntier != 5:
#            self.tier = params[ "TIER" ]
            self.tier = tier_name
        else:
            self.tier = ('A','B','C','D','E')
        
        self.blockLayers = [
            self.tech.getLayer("prBoundary"),
            ]
            
    def genLayout( self):
        
        #print IO_bank_cols
        #print Coord_X_IO
        #print Coord_Y_IO
        
        p = ParamArray()
        inst0 = InstanceArray('LVS_test/'+Via_name_bank, Via_pitch_bank, Via_pitch_bank, IO_bank_rows, IO_bank_cols, p, None, 'IO_bank')        
        inst0.setOrigin(Point(-Coord_X_IO+Via_size_bank/2, -Coord_Y_IO+Via_size_bank/2))
        #inst0.setOrigin(Point(0, 0))

        if wire_bank > max(Coord_X_IO, Coord_Y_IO):        
            Coord_X = width_rdec+width_ld+wire_bank
            Coord_Y = height_mux+height_sa+height_wd+wire_bank
            height_bank = height_sub*rows_sub
            width_bank = width_sub*cols_sub
        else:
            Coord_X = width_rdec+width_ld
            Coord_Y = height_mux+height_sa+height_wd
            height_bank = height_sub*rows_sub + Coord_Y_IO*2 - wire_bank*2
            width_bank = width_sub*cols_sub + Coord_X_IO*2 - wire_bank*2

        p = ParamArray()
        
        inst1 = InstanceArray('py_lib/cell_array', width_sub, height_sub, rows_sub/2, cols_sub/2, p, None, 'sub1')        
        inst1.setOrigin(Point(Coord_X+Coord_X_IO, Coord_Y+Coord_Y_IO))       
        inst2 = InstanceArray('py_lib/cell_array', width_sub, height_sub, rows_sub/2, cols_sub/2, p, None, 'sub2')
        #inst2.setOrientation(Orientation('R90'))
        inst2.setOrigin(Point(Coord_X+Coord_X_IO, Coord_Y+Coord_Y_IO))
        inst2.mirrorX()
        inst3 = InstanceArray('py_lib/cell_array', width_sub, height_sub, rows_sub/2, cols_sub/2, p, None, 'sub3')
        #inst3.setOrientation(R180)        
        inst3.setOrigin(Point(Coord_X+Coord_X_IO, Coord_Y+Coord_Y_IO))
        inst3.mirrorY()       
        inst4 = InstanceArray('py_lib/cell_array', width_sub, height_sub, rows_sub/2, cols_sub/2, p, None, 'sub4')
        #inst4.setOrientation(R270)
        inst4.setOrigin(Point(Coord_X+Coord_X_IO, Coord_Y+Coord_Y_IO))    
        inst4.mirrorX()
        inst4.mirrorY()
        
        #height_bank = height_sub*rows_sub + Coord_X_IO*2
        #width_bank = width_sub*cols_sub + Coord_Y_IO*2
        
        for lyr in self.blockLayers:
            Rect( lyr, Box( -width_bank/2, -height_bank/2, width_bank/2, height_bank/2))


height_bank = height_sub*rows_sub + Coord_Y_IO*2
width_bank = width_sub*cols_sub + Coord_X_IO*2

## define top IO dimension
IO_col_f= pins_top*1.0 / IO_top_rows
#print 'IO_col_f ='+str(IO_col_f)
IO_col_i=int(round(IO_col_f))
if IO_col_i == IO_col_f:
	IO_top_cols=IO_col_i
else:
	IO_top_cols=IO_col_i+1
	#IO_col_r= pins_top - IO_col_i*IO_top_rows	

Coord_X_IO_top = (Via_pitch_top * (IO_top_cols-1) + Via_size_top)/2
Coord_Y_IO_top = (Via_pitch_top * (IO_top_rows-1) + Via_size_top)/2

Y_offset_tp = 0

class top(squ3d):

    @classmethod
    def defineParamSpecs( cls, specs):
        squ3d.defineParamSpecs( specs)

        specs("IO_type", IO_type_top)
        #specs("Rows", IO_bank_rows)
        #specs("Cols", IO_bank_cols)

    def setupParams( self, params):
        squ3d.setupParams(self,params)
#        if params[ "TIER" ] != 'All':
        if Ntier != 5:
#            self.tier = params[ "TIER" ]
            self.tier = tier_name
        else:
            self.tier = ('A','B','C','D','E')
            
    def genLayout( self):
       
        Coord_X = width_bank+wire_top*2
        Coord_Y = height_bank+wire_top*2
        
        #if wire_top < Coord_X_IO_top:      
        #    IO_X_offset = Via_size_top/2+Coord_X*(cols_top/2-1)+width_bank/2
        #else:
        #    IO_X_offset = -Coord_X_IO_top+Via_size_top/2+Coord_X*(cols_top/2-1)+width_bank/2+wire_top
        if IO_position_top == 'E':
            IO_X_offset = Via_size_top/2-width_bank/2
        else:
            IO_X_offset = -Coord_X_IO_top+Via_size_top/2+Coord_X*(cols_top/2-1)+width_bank/2+wire_top*2
        
        if wire_top < Coord_Y_IO_top:      
            IO_Y_offset = Via_size_top/2+Coord_Y*(rows_top/2-1)+height_bank/2
        else:
            IO_Y_offset = -Coord_Y_IO_top+Via_size_top/2+Coord_Y*(rows_top/2-1)+height_bank/2+wire_top
        
        if wire_top*2 < Coord_Y_IO_top:
            Y_offset_tp = Coord_Y_IO_top*2 - wire_top*2
        else:
            Y_offset_tp = 0
        
        p = ParamArray()
        inst0 = InstanceArray('LVS_test/'+Via_name_top, Via_pitch_top, Via_pitch_top, IO_top_rows, IO_top_cols, p, None, 'IO_top')        
        inst0.setOrigin(Point(IO_X_offset, IO_Y_offset))
        #inst0.setOrigin(Point(0, 0))
        Pin_list = pingen()
        for i in range(IO_top_rows):
            for j in range(IO_top_cols):
                k = i*IO_top_cols + j
                X_Pin = j*Via_pitch_top + IO_X_offset
                Y_Pin = i*Via_pitch_top + IO_Y_offset
                if k<len(Pin_list):
                    text1 = Text(self.tech.getLayer("text"),Pin_list[k],Point(X_Pin,Y_Pin),Via_pitch_top/10)
       
        p = ParamArray()
        
        #for i in range(rows_top_NOB):        
        #inst1 = Instance('py_lib/bank', p, None, 'bank')      
        for i in range(rows_top):
            for j in range(cols_top):
              #print i, j
              inst1 = Instance('py_lib/bank', p, None, chr(i)+chr(j)) 
              if j >= rows_top/2:
                  #if wire_top*2 < Coord_X_IO_top:
                  #    X_offset = Coord_X_IO_top*2 - wire_top*2
                  #else:
                  #    X_offset = 0
                  X_offset = wire_top*2
              else:
                  X_offset = 0
              if i >= cols_top/2:
                  Y_offset = Y_offset_tp
#                  if wire_top*2 < Coord_Y_IO_top:
#                      Y_offset = Coord_Y_IO_top*2 - wire_top*2
#                  else:
#                      Y_offset = 0
              else:
                  Y_offset = 0
              inst1.setOrigin(Point(Coord_X*j+X_offset, Coord_Y*i+Y_offset))
                     

width_top = (width_bank+wire_top*2)*cols_top + wire_top*2
height_top = (height_bank+wire_top*2)*rows_top + Y_offset_tp


## delay calculation
# cal.py

#### Timing Information Output ####
# lib.py
