#######################################################################
# lib.py
#######################################################################
"""
   generate .lib file for memory array
"""

# PyCell code
from cni.dlo import (Box, DloGen, Instance, ParamArray, Point, Rect, Term, Tech, Layer, TermType, Pin, PropSet, InstanceArray, PhysicalComponent, Text, Orientation)
import math
from cell import (data_width, addr_width, width_top, height_top, Ntier)
from cal import (TDelay2)

#### Timing Information Output ####

#print '##########################################################################\n\n'
print 'CREATING LIB FILE FOR GIVEN MEMORY CONFIGURATION\n'

#data_width=output_bit*rows_top*cols_top
data_msb = data_width -1
#addr_width=rows_top_NOB + cols_top_NOB + rows_sub_NOB + cols_sub_NOB + rows_cell_NOB + cols_cell_NOB
#addr_width=rows_sub_NOB + cols_sub_NOB + rows_cell_NOB + cols_cell_NOB
addr_msb = addr_width -1
area = width_top * height_top
max_transition	=	2.8;
addr_cap	=	0.0139855;  #Borrowed from BUF4x input capacitance
en_cap	=	0.0125; #Borrowed from NAND2x input capacitance
data_cap	=	0.00882947; #Borrowed from DFF input capacitance
clk_cap	=	0.0373938;	#Borrowed from CLKBUF3 input capacitance

#decode_power = decode_data_dyn * 1000
#decode_power = 10
decode_power = 1*addr_width
"""
# dyn power calculation
$rd_data_power_dyn 	=	$decode_data_dyn + $wl_data_energy_dyn + ($rd_data_bl_dyn*$scalefactor) + ($op_data_driver_dyn*$scalefactor) + ($sa_data_dyn*$scalefactor);
$wr_data_power_dyn 	=	$decode_data_dyn + $wl_data_energy_dyn + ($wr_data_bl_dyn*$scalefactor);
$rd_tag_power_dyn 	=	$decode_tag_dyn + $wl_tag_energy_dyn + $rd_tag_bl_dyn + $sa_tag_dyn + $compare_dyn + $valid_dyn;
$wr_tag_power_dyn 	=	$decode_tag_dyn + $wl_tag_energy_dyn + $wr_tag_bl_dyn + $compare_dyn + $valid_dyn;

$rd_data_power_dyn	=	$rd_data_power_dyn + $scalefactor*(($tot_rd_energy_dyn - $rd_data_power_dyn - $rd_tag_power_dyn)* ($data_ram_area)/($data_ram_area + $tag_ram_area));
$wr_data_power_dyn	=	$wr_data_power_dyn + $scalefactor*(($tot_wr_energy_dyn - $wr_data_power_dyn - $wr_tag_power_dyn)* ($data_ram_area)/($data_ram_area + $tag_ram_area));
$rd_power_dyn = ($rd_data_power_dyn*1000);
$wr_power_dyn = ($wr_data_power_dyn*1000);
"""
rd_power_dyn = area * Ntier * 1.114e-5
wr_power_dyn = area * Ntier * 4.432e-6
leakage = area * Ntier * 3.65e-8

access_time = TDelay2 / 1000

addr_rise_base = [0.0510, 0.0515, 0.0522, 0.0550, 0.0604, 0.0700, 0.0893]
addr_fall_base = [0.0191, 0.0203, 0.0216, 0.0275, 0.0391, 0.0592, 0.0997]
addr_rise=[0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0]
addr_fall=[0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0]
driver_delay = [0.0094, 0.0108, 0.0132, 0.0220, 0.0352, 0.0572, 0.0924]
output_delay=[0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0]
op_fall_tr = [0.0046281, 0.0050589, 0.00804, 0.0196, 0.0378, 0.0574, 0.0952]
op_rise_tr=[0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0]
for i in range(0,7):
    addr_rise[i] = addr_rise_base[i]*addr_width
    addr_fall[i] = addr_fall_base[i]*addr_width
    output_delay[i] = access_time + driver_delay[i]
    op_rise_tr[i] = op_fall_tr[i]*3

ofilename = './mem_'+str(data_width)+'_'+str(addr_width)+'.lib'

dest=file(ofilename,'w')

def writeDelay(dest):
    dest.write(' \
library(MemGen_'+str(data_width)+'_'+str(addr_width)+') { \n\
	delay_model		: table_lookup; \n\
	time_unit		: "1ns"; \n\
	voltage_unit		: "1V"; \n\
	current_unit		: "1mA"; \n\
  	leakage_power_unit      : "1mW"; \n\
	nom_process		: 1; \n\
	nom_temperature		: 25.000; \n\
	nom_voltage		: 1.000; \n\
	capacitive_load_unit	 (1,pf); \n\
 \n\
	pulling_resistance_unit	        : "1kohm"; \n\
 \n\
	/* additional header data */ \n\
	default_cell_leakage_power  : 0; \n\
	default_fanout_load			: 1; \n\
	default_inout_pin_cap		: 0.005; \n\
	default_input_pin_cap		: 0.005; \n\
	default_output_pin_cap		: 0.0; \n\
	default_max_transition		: '+str(max_transition)+';  \n\
 \n\
    /* default attributes */ \n\
    default_leakage_power_density : 0.0; \n\
    slew_derate_from_library      : 1; \n\
    slew_lower_threshold_pct_fall : 10.000; \n\
    slew_upper_threshold_pct_fall : 90.000; \n\
    slew_lower_threshold_pct_rise : 10.000; \n\
    slew_upper_threshold_pct_rise : 90.000; \n\
    input_threshold_pct_fall      : 50.000; \n\
    input_threshold_pct_rise      : 50.000; \n\
    output_threshold_pct_fall     : 50.000; \n\
    output_threshold_pct_rise     : 50.000; \n\
 \n\
 	/* k-factors */ \n\
 	k_process_cell_fall             : 1; \n\
 	k_process_cell_leakage_power    : 0; \n\
 	k_process_cell_rise             : 1; \n\
 	k_process_fall_transition       : 1; \n\
 	k_process_hold_fall             : 1; \n\
 	k_process_hold_rise             : 1; \n\
 	k_process_internal_power        : 0; \n\
 	k_process_min_pulse_width_high  : 1; \n\
 	k_process_min_pulse_width_low   : 1; \n\
 	k_process_pin_cap               : 0; \n\
 	k_process_recovery_fall         : 1; \n\
 	k_process_recovery_rise         : 1; \n\
 	k_process_rise_transition       : 1; \n\
 	k_process_setup_fall            : 1; \n\
 	k_process_setup_rise            : 1; \n\
 	k_process_wire_cap              : 0; \n\
 	k_process_wire_res              : 0; \n\
	k_temp_cell_fall		: 0.0; \n\
	k_temp_cell_rise		: 0.0; \n\
	k_temp_hold_fall                : 0.0; \n\
	k_temp_hold_rise                : 0.0; \n\
	k_temp_min_pulse_width_high     : 0.0; \n\
	k_temp_min_pulse_width_low      : 0.0; \n\
	k_temp_min_period               : 0.0; \n\
	k_temp_rise_propagation         : 0.0; \n\
	k_temp_fall_propagation         : 0.0; \n\
	k_temp_rise_transition          : 0.0; \n\
	k_temp_fall_transition          : 0.0; \n\
	k_temp_recovery_fall            : 0.0; \n\
	k_temp_recovery_rise            : 0.0; \n\
	k_temp_setup_fall               : 0.0; \n\
	k_temp_setup_rise               : 0.0; \n\
	k_volt_cell_fall                : 0.0; \n\
	k_volt_cell_rise                : 0.0; \n\
	k_volt_hold_fall                : 0.0; \n\
	k_volt_hold_rise                : 0.0; \n\
	k_volt_min_pulse_width_high     : 0.0; \n\
	k_volt_min_pulse_width_low      : 0.0; \n\
	k_volt_min_period               : 0.0; \n\
	k_volt_rise_propagation         : 0.0; \n\
	k_volt_fall_propagation         : 0.0; \n\
	k_volt_rise_transition	        : 0.0; \n\
	k_volt_fall_transition	        : 0.0; \n\
	k_volt_recovery_fall            : 0.0; \n\
	k_volt_recovery_rise            : 0.0; \n\
	k_volt_setup_fall               : 0.0; \n\
	k_volt_setup_rise               : 0.0; \n\
 \n\
 \n\
	operating_conditions(typical) { \n\
		process		 : 1; \n\
		temperature	 : 25.000; \n\
		voltage		 : 1.000; \n\
		tree_type	 : balanced_tree; \n\
	} \n\
 \n\
	wire_load("sample") { \n\
		resistance	 : 1.6e-05; \n\
		capacitance	 : 0.0002; \n\
		area		 : 1.7; \n\
  		slope		 : 500; \n\
		fanout_length	 (1,500); \n\
	} \n\
    lu_table_template(mem_delay_template) { \n\
       variable_1 : total_output_net_capacitance; \n\
           index_1 ("1000, 1001, 1002, 1003, 1004, 1005, 1006"); \n\
    } \n\
	lu_table_template(mem_constraint_template_1) { \n\
           variable_1 : related_pin_transition; \n\
               index_1 ("1000, 1001, 1002, 1003, 1004, 1005, 1006"); \n\
        } \n\
	lu_table_template(mem_constraint_template_2) { \n\
           variable_1 : constrained_pin_transition; \n\
               index_1 ("1000, 1001, 1002, 1003, 1004, 1005, 1006"); \n\
        } \n\
	lu_table_template(mem_load_template) { \n\
           variable_1 : total_output_net_capacitance; \n\
               index_1 ("1000, 1001, 1002, 1003, 1004, 1005, 1006"); \n\
        } \n\
    power_lut_template(mem_passive_energy_template) { \n\
       variable_1 : input_transition_time; \n\
           index_1 ("1000, 1001"); \n\
    } \n\
	library_features(report_delay_calculation); \n\
	type (DATA_BUS) { \n\
		base_type : array ; \n\
		data_type : bit ; \n\
		bit_width : '+str(data_width)+'; \n\
		bit_from  : '+str(data_msb)+'; \n\
		bit_to	  : 0 ; \n\
		downto 	  : true ; \n\
	} \n\
 \n\
	type (ADDR_BUS) { \n\
		base_type : array ; \n\
		data_type : bit ; \n\
		bit_width : '+str(addr_width)+'; \n\
		bit_from  : '+str(addr_msb)+'; \n\
		bit_to    : 0 ; \n\
		downto    : true ; \n\
	} \n\
 \n\
	 \n\
cell(MemGen_'+str(data_width)+'_'+str(addr_width)+') { \n\
	area		 : '+str(area)+'; \n\
	dont_use	 : TRUE; \n\
	dont_touch	 : TRUE; \n\
	interface_timing : TRUE; \n\
	memory() { \n\
		type : ram; \n\
		address_width : '+str(addr_width)+'; \n\
		word_width : '+str(data_width)+'; \n\
	} \n\
 \n\
	/* 	******************************************************************* \n\
		MEMORY WRITE INTERFACE: CENB (wr_en), AB (wr_addr), DB (wr data), CLKB \n\
	********************************************************************* */ \n\
	pin(wr_en) { \n\
		direction : input; \n\
		capacitance : '+str(en_cap)+'; \n\
		timing() { \n\
			related_pin : "clock_rw" ; \n\
			timing_type : setup_rising ; \n\
			rise_constraint(mem_constraint_template_1) { \n\
			/* 	************************************************************ \n\
				CHANGES WITH RELATED PIN (CLK) TRANSITION \n\
			************************************************************** */ \n\
				index_1 ("0.010, 0.050, 0.100, 0.300, 0.700, 1.400, 2.800"); \n\
				values ( "0.256, 0.258, 0.262, 0.275, 0.302, 0.349, 0.446") \n\
				 \n\
			} \n\
			fall_constraint(mem_constraint_template_2) { \n\
			/* 	************************************************************ \n\
				CHANGES WITH CONSTRAINED PIN TRANSITION \n\
			************************************************************** */ \n\
				index_1 ("0.010, 0.050, 0.100, 0.300, 0.700, 1.400, 2.800"); \n\
				values ( "0.506, 0.513, 0.520, 0.550, 0.609, 0.712, 0.920") \n\
			}	 \n\
		} \n\
		timing() { \n\
			related_pin : "clock_rw" ; \n\
			timing_type : hold_rising ; \n\
			rise_constraint(mem_constraint_template_2) { \n\
			/* 	************************************************************ \n\
				NO HOLD CONSTRATINT FOR CENB W.R.T CLKB \n\
			************************************************************** */ \n\
				index_1 ("0.010, 0.050, 0.100, 0.300, 0.700, 1.400, 2.800"); \n\
				values  ("0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000") \n\
			} \n\
			fall_constraint(mem_constraint_template_2) { \n\
			/* 	************************************************************ \n\
				NO HOLD CONSTRATINT FOR CENB W.R.T CLKB \n\
			************************************************************** */ \n\
				index_1 ("0.010, 0.050, 0.100, 0.300, 0.700, 1.400, 2.800"); \n\
				values  ("0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000") \n\
			}	 \n\
		}	 \n\
	} \n\
 \n\
')
## remove bus 'wr_addr' (write address input)
#	bus (wr_addr)  { \n\
#		bus_type : ADDR_BUS; \n\
#  	    direction : input; \n\
#		capacitance : '+str(addr_cap)+'; \n\
#	    internal_power(){ \n\
#	    	when : "!wr_en"; \n\
#			power(mem_passive_energy_template) { \n\
#				index_1 ("0.0 1.0"); \n\
#	    		values ("'+str(decode_power)+', '+str(decode_power)+'")  \n\
#	    		/* Decoding Power*/ \n\
#			}	 \n\
#		} \n\
#		timing() { \n\
#			related_pin : "clock_rw" \n\
#			timing_type : setup_rising ; \n\
#			rise_constraint(mem_constraint_template_2) { \n\
#				index_1 ("0.010, 0.050, 0.100, 0.300, 0.700, 1.400, 2.800"); \n\
#				values  ("'+str(addr_rise[0])+', '+str(addr_rise[1])+', '+str(addr_rise[2])+', '+str(addr_rise[3])+', '+str(addr_rise[4])+', '+str(addr_rise[5])+', '+str(addr_rise[6])+'") \n\
#				/* 	************************************************************ \n\
#					CHANGES WITH CONSTRAINED PIN TRANSITION \n\
#				************************************************************** */ \n\
#			} \n\
#			fall_constraint(mem_constraint_template_2) { \n\
#				/* 	************************************************************ \n\
#					CHANGES WITH CONSTRAINED PIN TRANSITION \n\
#				************************************************************** */ \n\
#				index_1 ("0.010, 0.050, 0.100, 0.300, 0.700, 1.400, 2.800"); \n\
#				values  ("'+str(addr_fall[0])+', '+str(addr_fall[1])+', '+str(addr_fall[2])+', '+str(addr_fall[3])+', '+str(addr_fall[4])+', '+str(addr_fall[5])+', '+str(addr_fall[6])+'") \n\
#			}	 \n\
#		} \n\
#		timing() { \n\
#			related_pin : "clock_rw" \n\
#			timing_type : hold_rising ; \n\
#			rise_constraint(mem_constraint_template_2) { \n\
#			/* 	************************************************************ \n\
#				NO HOLD CONSTRATINT FOR AB W.R.T CLKB \n\
#			************************************************************** */ \n\
#				index_1 ("0.010, 0.050, 0.100, 0.300, 0.700, 1.400, 2.800"); \n\
#				values  ("0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000") \n\
#			} \n\
#			fall_constraint(mem_constraint_template_2) { \n\
#			/* 	************************************************************ \n\
#				NO HOLD CONSTRATINT FOR AB W.R.T CLKB \n\
#			************************************************************** */ \n\
#				index_1 ("0.010, 0.050, 0.100, 0.300, 0.700, 1.400, 2.800"); \n\
#				values  ("0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000") \n\
#			}	 \n\
#		}	 \n\
#	} \n\
# \n\

    dest.write(' \
	bus(wr_data)	 { \n\
		bus_type : DATA_BUS; \n\
		direction : input; \n\
		capacitance : '+str(data_cap)+'; \n\
		memory_write() { \n\
			address : rd_addr1; \n\
			clocked_on : "clock_rw"; \n\
		} \n\
		timing() { \n\
			related_pin : "clock_rw" \n\
			timing_type : setup_rising ; \n\
			rise_constraint(mem_constraint_template_1) { \n\
			/* 	************************************************************ \n\
				CHANGES WITH RELATED PIN \n\
			************************************************************** */ \n\
				index_1 ("0.010, 0.050, 0.100, 0.300, 0.700, 1.400, 2.800"); \n\
				values  ("0.036, 0.039, 0.041, 0.053, 0.077, 0.119, 0.203") \n\
				 \n\
			} \n\
			fall_constraint(mem_constraint_template_2) { \n\
				index_1 ("0.010, 0.050, 0.100, 0.300, 0.700, 1.400, 2.800"); \n\
				/* 	************************************************************ \n\
					CHANGES WITH CONSTRAINED PIN \n\
				************************************************************** */ \n\
				values  ("0.077, 0.084, 0.094, 0.131, 0.206, 0.337, 0.599") \n\
			} \n\
		} \n\
		timing() { \n\
			related_pin : "clock_rw" \n\
			timing_type : hold_rising ; \n\
			rise_constraint(mem_constraint_template_1) { \n\
			/* 	************************************************************ \n\
				CHANGES WITH RELATED PIN \n\
			************************************************************** */ \n\
				index_1 ("0.010, 0.050, 0.100, 0.300, 0.700, 1.400, 2.800"); \n\
				values  ("0.232, 0.230, 0.227, 0.216, 0.194, 0.156, 0.078")				 \n\
			} \n\
			fall_constraint(mem_constraint_template_2) { \n\
				values  ("0.186, 0.184, 0.181, 0.170, 0.148, 0.109, 0.032") \n\
			}	 \n\
		}	 \n\
	} \n\
 \n\
')
## remove pin 'clock_wr' (write clock input)
#	pin(clock_wr) { \n\
#		direction : input; \n\
#		capacitance : '+str(clk_cap)+'; \n\
#		clock	 : true; \n\
#		min_pulse_width_low	 : 0.211; \n\
#		min_pulse_width_high	 : 0.058; \n\
#		min_period		 : 	'+str(access_time)+';  \n\
#        max_transition           : '+str(max_transition)+'; \n\
#	    internal_power(){ \n\
#	    	when : "wr_en"; \n\
#			power(mem_passive_energy_template) { \n\
#	    		index_1 ("0.0 1.0"); \n\
#	    		values  ("0.0, 0.0") \n\
#			} \n\
#		} \n\
#	    internal_power(){ \n\
#			when : "!wr_en "; \n\
#			rise_power(mem_passive_energy_template) { \n\
#	    		index_1 ("0.0 1.0"); \n\
#	    		values  ("'+str(wr_power_dyn)+', '+str(wr_power_dyn)+'") \n\
#	    		/* */ \n\
#			}	 \n\
#			fall_power(mem_passive_energy_template) { \n\
#	    		index_1 ("0.0 1.0"); \n\
#	    		values  ("0.0, 0.0") \n\
#			}	 \n\
#		}	 \n\
#	} \n\
# \n\
#		 \n\

    dest.write(' \
	pin(rd_en1) { \n\
		direction : input; \n\
		capacitance : '+str(en_cap)+'; \n\
		timing() { \n\
			related_pin : "clock_rw" ; \n\
			timing_type : setup_rising ; \n\
			rise_constraint(mem_constraint_template_1) { \n\
			/* 	************************************************************ \n\
				CHANGES WITH RELATED PIN (CLK) TRANSITION \n\
			************************************************************** */ \n\
				index_1 ("0.010, 0.050, 0.100, 0.300, 0.700, 1.400, 2.800"); \n\
				values ( "0.256, 0.258, 0.262, 0.275, 0.302, 0.349, 0.446") \n\
				 \n\
			} \n\
			fall_constraint(mem_constraint_template_2) { \n\
			/* 	************************************************************ \n\
				CHANGES WITH CONSTRAINED PIN TRANSITION \n\
			************************************************************** */ \n\
				index_1 ("0.010, 0.050, 0.100, 0.300, 0.700, 1.400, 2.800"); \n\
				values ( "0.506, 0.513, 0.520, 0.550, 0.609, 0.712, 0.920") \n\
			}	 \n\
		} \n\
		timing() { \n\
			related_pin : "clock_rw" ; \n\
			timing_type : hold_rising ; \n\
			rise_constraint(mem_constraint_template_2) { \n\
			/* 	************************************************************ \n\
				NO HOLD CONSTRATINT FOR CENB W.R.T CLKB \n\
			************************************************************** */ \n\
				index_1 ("0.010, 0.050, 0.100, 0.300, 0.700, 1.400, 2.800"); \n\
				values  ("0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000") \n\
			} \n\
			fall_constraint(mem_constraint_template_2) { \n\
			/* 	************************************************************ \n\
				NO HOLD CONSTRATINT FOR CENB W.R.T CLKB \n\
			************************************************************** */ \n\
				index_1 ("0.010, 0.050, 0.100, 0.300, 0.700, 1.400, 2.800"); \n\
				values  ("0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000") \n\
			}	 \n\
		}	 \n\
	} \n\
 \n\
	bus(rd_addr1)  { \n\
		bus_type : ADDR_BUS; \n\
  	    direction : input; \n\
		capacitance : '+str(addr_cap)+'; \n\
	    internal_power(){ \n\
	    	when : "!rd_en1"; \n\
			power(mem_passive_energy_template) { \n\
				index_1 ("0.0 1.0"); \n\
	    		values ("'+str(decode_power)+', '+str(decode_power)+'")  \n\
	    		/* Decoding Power*/ \n\
			}	 \n\
		} \n\
		timing() { \n\
			related_pin : "clock_rw" \n\
			timing_type : setup_rising ; \n\
			rise_constraint(mem_constraint_template_2) { \n\
				index_1 ("0.010, 0.050, 0.100, 0.300, 0.700, 1.400, 2.800"); \n\
				values  ("'+str(addr_rise[0])+', '+str(addr_rise[1])+', '+str(addr_rise[2])+', '+str(addr_rise[3])+', '+str(addr_rise[4])+', '+str(addr_rise[5])+', '+str(addr_rise[6])+'") \n\
				/* 	************************************************************ \n\
					CHANGES WITH CONSTRAINED PIN TRANSITION \n\
				************************************************************** */ \n\
			} \n\
			fall_constraint(mem_constraint_template_2) { \n\
				/* 	************************************************************ \n\
					CHANGES WITH CONSTRAINED PIN TRANSITION \n\
				************************************************************** */ \n\
				index_1 ("0.010, 0.050, 0.100, 0.300, 0.700, 1.400, 2.800"); \n\
				values  ("'+str(addr_fall[0])+', '+str(addr_fall[1])+', '+str(addr_fall[2])+', '+str(addr_fall[3])+', '+str(addr_fall[4])+', '+str(addr_fall[5])+', '+str(addr_fall[6])+'") \n\
			}	 \n\
		} \n\
		timing() { \n\
			related_pin : "clock_rw" \n\
			timing_type : hold_rising ; \n\
			rise_constraint(mem_constraint_template_2) { \n\
			/* 	************************************************************ \n\
				NO HOLD CONSTRATINT FOR AB W.R.T CLKB \n\
			************************************************************** */ \n\
				index_1 ("0.010, 0.050, 0.100, 0.300, 0.700, 1.400, 2.800"); \n\
				values  ("0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000") \n\
			} \n\
			fall_constraint(mem_constraint_template_2) { \n\
			/* 	************************************************************ \n\
				NO HOLD CONSTRATINT FOR AB W.R.T CLKB \n\
			************************************************************** */ \n\
				index_1 ("0.010, 0.050, 0.100, 0.300, 0.700, 1.400, 2.800"); \n\
				values  ("0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000") \n\
			}	 \n\
		}	 \n\
	} \n\
 \n\
	 \n\
	bus(rd_data1) { \n\
      	bus_type : DATA_BUS; \n\
        direction : output; \n\
        memory_read() { \n\
			address : rd_addr1; \n\
		} \n\
		timing() { \n\
			related_pin :	"clock_rw"; \n\
			timing_type : rising_edge; \n\
			timing_sense : non_unate; \n\
 \n\
			cell_rise(mem_delay_template) {  \n\
			/* 	************************************************************ \n\
				CHANGES WITH OUTPUT NET CAPACITANCE \n\
			************************************************************** */ \n\
				index_1 ("0.000, 0.050, 0.100, 0.200, 0.370, 0.750, 1.480"); \n\
				values  ("'+str(output_delay[0])+', '+str(output_delay[1])+', '+str(output_delay[2])+', '+str(output_delay[3])+', '+str(output_delay[4])+', '+str(output_delay[5])+', '+str(output_delay[6])+'") \n\
			} \n\
			cell_fall(mem_delay_template) { \n\
			/* 	************************************************************ \n\
				CHANGES WITH OUTPUT NET CAPACITANCE \n\
			************************************************************** */ \n\
				index_1 ("0.000, 0.050, 0.100, 0.200, 0.370, 0.750, 1.480"); \n\
				values  ("'+str(output_delay[0])+', '+str(output_delay[1])+', '+str(output_delay[2])+', '+str(output_delay[3])+', '+str(output_delay[4])+', '+str(output_delay[5])+', '+str(output_delay[6])+'") \n\
			} \n\
			 \n\
			rise_transition(mem_load_template) { \n\
				index_1 ("0.000, 0.050, 0.100, 0.200, 0.370, 0.750, 1.480"); \n\
				values  ("'+str(op_rise_tr[0])+', '+str(op_rise_tr[1])+', '+str(op_rise_tr[2])+', '+str(op_rise_tr[3])+', '+str(op_rise_tr[4])+', '+str(op_rise_tr[5])+', '+str(op_rise_tr[6])+'") \n\
			} \n\
			fall_transition(mem_load_template) { \n\
				index_1 ("0.000, 0.050, 0.100, 0.200, 0.370, 0.750, 1.480"); \n\
				values  ("'+str(op_fall_tr[0])+', '+str(op_fall_tr[1])+', '+str(op_fall_tr[2])+', '+str(op_fall_tr[3])+', '+str(op_fall_tr[4])+', '+str(op_fall_tr[5])+', '+str(op_fall_tr[6])+'") \n\
			}	 \n\
		} \n\
	} \n\
	 \n\
	pin(clock_rw) { \n\
		direction : input; \n\
		capacitance : '+str(clk_cap)+'; \n\
		clock	 : true; \n\
		min_pulse_width_low	 : 0.211; \n\
		min_pulse_width_high	 : 0.058; \n\
		min_period		 : '+str(access_time)+';  \n\
        max_transition           : '+str(max_transition)+'; \n\
	    internal_power(){ \n\
	    	when : "rd_en1"; \n\
			rise_power(mem_passive_energy_template) { \n\
	    		index_1 ("0.0 1.0"); \n\
	    		values  ("'+str(wr_power_dyn)+', '+str(wr_power_dyn)+'") \n\
	    		/* */ \n\
			}	 \n\
			fall_power(mem_passive_energy_template) { \n\
	    		index_1 ("0.0 1.0"); \n\
	    		values  ("0.0, 0.0") \n\
			} \n\
		} \n\
	    internal_power(){ \n\
			when : "!rd_en1 "; \n\
			rise_power(mem_passive_energy_template) { \n\
	    		index_1 ("0.0 1.0"); \n\
	    		values  ("'+str(rd_power_dyn)+', '+str(rd_power_dyn)+'") \n\
	    		/* */ \n\
			}	 \n\
			fall_power(mem_passive_energy_template) { \n\
	    		index_1 ("0.0 1.0"); \n\
	    		values  ("0.0, 0.0") \n\
			}	 \n\
		}	 \n\
	} \n\
 \n\
')
## remove pin 'rd_en2' (2nd read enable)
#	pin(rd_en2) { \n\
#		direction : input; \n\
#		capacitance : '+str(en_cap)+'; \n\
#		timing() { \n\
#			related_pin : "clock_rd2" ; \n\
#			timing_type : setup_rising ; \n\
#			rise_constraint(mem_constraint_template_1) { \n\
#			/* 	************************************************************ \n\
#				CHANGES WITH RELATED PIN (CLK) TRANSITION \n\
#			************************************************************** */ \n\
#				index_1 ("0.010, 0.050, 0.100, 0.300, 0.700, 1.400, 2.800"); \n\
#				values ( "0.256, 0.258, 0.262, 0.275, 0.302, 0.349, 0.446") \n\
#				 \n\
#			} \n\
#			fall_constraint(mem_constraint_template_2) { \n\
#			/* 	************************************************************ \n\
#				CHANGES WITH CONSTRAINED PIN TRANSITION \n\
#			************************************************************** */ \n\
#				index_1 ("0.010, 0.050, 0.100, 0.300, 0.700, 1.400, 2.800"); \n\
#				values ( "0.506, 0.513, 0.520, 0.550, 0.609, 0.712, 0.920") \n\
#			}	 \n\
#		} \n\
#		timing() { \n\
#			related_pin : "clock_rd2" ; \n\
#			timing_type : hold_rising ; \n\
#			rise_constraint(mem_constraint_template_2) { \n\
#			/* 	************************************************************ \n\
#				NO HOLD CONSTRATINT FOR CENB W.R.T CLKB \n\
#			************************************************************** */ \n\
#				index_1 ("0.010, 0.050, 0.100, 0.300, 0.700, 1.400, 2.800"); \n\
#				values  ("0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000") \n\
#			} \n\
#			fall_constraint(mem_constraint_template_2) { \n\
#			/* 	************************************************************ \n\
#				NO HOLD CONSTRATINT FOR CENB W.R.T CLKB \n\
#			************************************************************** */ \n\
#				index_1 ("0.010, 0.050, 0.100, 0.300, 0.700, 1.400, 2.800"); \n\
#				values  ("0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000") \n\
#			}	 \n\
#		}	 \n\
#	} \n\
# \n\

## remove bus 'rd_addr2' (2nd read address input bus)
#	bus (rd_addr2)  { \n\
#		bus_type : ADDR_BUS; \n\
#  	    direction : input; \n\
#		capacitance : '+str(addr_cap)+'; \n\
#	    internal_power(){ \n\
#	    	when : "!rd_en2"; \n\
#			power(mem_passive_energy_template) { \n\
#				index_1 ("0.0 1.0"); \n\
#	    		values ("'+str(decode_power)+', '+str(decode_power)+'")  \n\
#	    		/* Decoding Power*/ \n\
#			}	 \n\
#		} \n\
#		timing() { \n\
#			related_pin : "clock_rd2" \n\
#			timing_type : setup_rising ; \n\
#			rise_constraint(mem_constraint_template_2) { \n\
#				index_1 ("0.010, 0.050, 0.100, 0.300, 0.700, 1.400, 2.800"); \n\
#				values  ("'+str(addr_rise[0])+', '+str(addr_rise[1])+', '+str(addr_rise[2])+', '+str(addr_rise[3])+', '+str(addr_rise[4])+', '+str(addr_rise[5])+', '+str(addr_rise[6])+'") \n\
#				/* 	************************************************************ \n\
#					CHANGES WITH CONSTRAINED PIN TRANSITION \n\
#				************************************************************** */ \n\
#			} \n\
#			fall_constraint(mem_constraint_template_2) { \n\
#				/* 	************************************************************ \n\
#					CHANGES WITH CONSTRAINED PIN TRANSITION \n\
#				************************************************************** */ \n\
#				index_1 ("0.010, 0.050, 0.100, 0.300, 0.700, 1.400, 2.800"); \n\
#				values  ("'+str(addr_fall[0])+', '+str(addr_fall[1])+', '+str(addr_fall[2])+', '+str(addr_fall[3])+', '+str(addr_fall[4])+', '+str(addr_fall[5])+', '+str(addr_fall[6])+'") \n\
#			}	 \n\
#		} \n\
#		timing() { \n\
#			related_pin : "clock_rd2" \n\
#			timing_type : hold_rising ; \n\
#			rise_constraint(mem_constraint_template_2) { \n\
#			/* 	************************************************************ \n\
#				NO HOLD CONSTRATINT FOR AB W.R.T CLKB \n\
#			************************************************************** */ \n\
#				index_1 ("0.010, 0.050, 0.100, 0.300, 0.700, 1.400, 2.800"); \n\
#				values  ("0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000") \n\
#			} \n\
#			fall_constraint(mem_constraint_template_2) { \n\
#			/* 	************************************************************ \n\
#				NO HOLD CONSTRATINT FOR AB W.R.T CLKB \n\
#			************************************************************** */ \n\
#				index_1 ("0.010, 0.050, 0.100, 0.300, 0.700, 1.400, 2.800"); \n\
#				values  ("0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000") \n\
#			}	 \n\
#		}	 \n\
#	} \n\
# \n\
## remove bus 'rd_data2' (2nd read data bus)
#	bus(rd_data2) { \n\
#      	bus_type : DATA_BUS; \n\
#        direction : output; \n\
#        memory_read() { \n\
#			address : rd_addr2; \n\
#		} \n\
#		timing() { \n\
#			related_pin :	"clock_rd2"; \n\
#			timing_type : rising_edge; \n\
#			timing_sense : non_unate; \n\
# \n\
#			cell_rise(mem_delay_template) {  \n\
#			/* 	************************************************************ \n\
#				CHANGES WITH OUTPUT NET CAPACITANCE \n\
#			************************************************************** */ \n\
#				index_1 ("0.000, 0.050, 0.100, 0.200, 0.370, 0.750, 1.480"); \n\
#				values  ("'+str(output_delay[0])+', '+str(output_delay[1])+', '+str(output_delay[2])+', '+str(output_delay[3])+', '+str(output_delay[4])+', '+str(output_delay[5])+', '+str(output_delay[6])+'") \n\
#			} \n\
#			cell_fall(mem_delay_template) { \n\
#			/* 	************************************************************ \n\
#				CHANGES WITH OUTPUT NET CAPACITANCE \n\
#			************************************************************** */ \n\
#				index_1 ("0.000, 0.050, 0.100, 0.200, 0.370, 0.750, 1.480"); \n\
#				values  ("'+str(output_delay[0])+', '+str(output_delay[1])+', '+str(output_delay[2])+', '+str(output_delay[3])+', '+str(output_delay[4])+', '+str(output_delay[5])+', '+str(output_delay[6])+'") \n\
#			} \n\
#			 \n\
#			rise_transition(mem_load_template) { \n\
#				index_1 ("0.000, 0.050, 0.100, 0.200, 0.370, 0.750, 1.480"); \n\
#				values  ("'+str(op_rise_tr[0])+', '+str(op_rise_tr[1])+', '+str(op_rise_tr[2])+', '+str(op_rise_tr[3])+', '+str(op_rise_tr[4])+', '+str(op_rise_tr[5])+', '+str(op_rise_tr[6])+'") \n\
#			} \n\
#			fall_transition(mem_load_template) { \n\
#				index_1 ("0.000, 0.050, 0.100, 0.200, 0.370, 0.750, 1.480"); \n\
#				values  ("'+str(op_fall_tr[0])+', '+str(op_fall_tr[1])+', '+str(op_fall_tr[2])+', '+str(op_fall_tr[3])+', '+str(op_fall_tr[4])+', '+str(op_fall_tr[5])+', '+str(op_fall_tr[6])+'") \n\
#			}	 \n\
#		} \n\
#	} \n\
#			 \n\
## remove pin 'clock_rd2' (2nd read clock pin)
#	pin(clock_rd2) { \n\
#		direction : input; \n\
#		capacitance : '+str(clk_cap)+'; \n\
#		clock	 : true; \n\
#		min_pulse_width_low	 : 0.211; \n\
#		min_pulse_width_high	 : 0.058; \n\
#		min_period		 : '+str(access_time)+';  \n\
#        max_transition           : '+str(max_transition)+'; \n\
#	    internal_power(){ \n\
#	    	when : "rd_en2"; \n\
#			power(mem_passive_energy_template) { \n\
#	    		index_1 ("0.0 1.0"); \n\
#	    		values  ("0.0, 0.0") \n\
#			} \n\
#		} \n\
#	    internal_power(){ \n\
#			when : "!rd_en2 "; \n\
#			rise_power(mem_passive_energy_template) { \n\
#	    		index_1 ("0.0 1.0"); \n\
#	    		values  ("'+str(rd_power_dyn)+', '+str(rd_power_dyn)+'") \n\
#	    		/* */ \n\
#			}	 \n\
#			fall_power(mem_passive_energy_template) { \n\
#	    		index_1 ("0.0 1.0"); \n\
#	    		values  ("0.0, 0.0") \n\
#			}	 \n\
#		}	 \n\
#	} \n\
# \n\

    dest.write(' \
	cell_leakage_power : '+str(leakage)+'; \n\
    } \n\
} \n\
')
writeDelay(dest)
